package it.mokabyte.pattern.client;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import java.rmi.RemoteException;

import it.mokabyte.pattern.ejb.session.stateless.Copy;
import it.mokabyte.pattern.ejb.session.stateless.CopyHome;
import it.mokabyte.pattern.pk.AccountPK;
import it.mokabyte.pattern.valueobject.AccountOM;

/**
 * <p>Title:CopyTestClient </p>
 * <p>Description: simple test for Copy EJB SL </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 */
public class CopyTestClient {

    /** EJB JNDI name */
    private final static String EJB_NAME = "MokaCopy";

    /** main */
    public static void main(String[] args) {
        System.out.println("starting CopyTestClient ...");
        new CopyTestClient().test();
    }

    /** Test the Entity EJB Account */
    public void test() {
      try {
          Context ctx = null;
          ctx =  new InitialContext();
          System.out.println("Lookup for ["+this.EJB_NAME+"]...");
          //look up jndi name
          Object obj = ctx.lookup(this.EJB_NAME);
          System.out.println("Lookup done : obj class["+obj.getClass().getName()+"] ...");
          System.out.println("Lookup done : obj ["+obj+"] ...");
          //cast to Home interface
          CopyHome copyHome = (CopyHome)PortableRemoteObject.narrow(obj, CopyHome.class);
          System.out.println("Narrow done ...accountHome["+copyHome+"]...");
          // start the test
          Copy copy = copyHome.create();

          String key = "001";

          // Account of PK will be update from JDBC to XML
          AccountPK pk = new AccountPK(key);
          System.out.println("Copying Account of primary key ["+key+"] from XML to JDBC ...");
          copy.copyAccount(pk);
          System.out.println("... the copy has been done !");

          AccountOM res_1 = copy.readAccountJDBC(pk);
          System.out.println("Account XML : " + res_1);

          AccountOM res_2 = copy.readAccountJDBC(pk);
          System.out.println("Account JDBC : " + res_2);

          if(res_1.equals(res_2)){
              System.out.println("Test ok ! Objects are equals !");
          }
          else {
              System.out.println(" ### Test failed ! Objects not equals ! ###");
          }

         } catch(ClassCastException cce) {
            System.out.println(" # ClassCastExceptionException : " + cce.getMessage());
            cce.printStackTrace();
        } catch(NamingException ne) {
            System.out.println(" # NamingException : " + ne.getMessage());
            ne.printStackTrace();
        } catch(CreateException ce) {
            System.out.println(" # CreateException : " + ce.getMessage());
            ce.printStackTrace();
        } catch(RemoteException re) {
            System.out.println(" # RemoteException : " + re.getMessage());
            re.printStackTrace();
        }
        catch(Exception e) {
            System.out.println(" # Exception  : " + e.getMessage());
            e.printStackTrace();
        }
    }
}