package it.mokabyte.pattern.ejb.entity;

import java.util.*;
import javax.ejb.*;

import it.mokabyte.pattern.dao.MyDao;
// import it.mokabyte.pattern.dao.MyDaoJdbcImpl; no more !
import it.mokabyte.pattern.dao.MyDaoFactory;
import it.mokabyte.pattern.valueobject.AccountOM;
import it.mokabyte.pattern.pk.AccountPK;
import it.mokabyte.pattern.exception.DAOException;

/**
 *
 * <p>Title: AccountBean </p>
 * <p>Description: a Bank Account BMP</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0 : was used MyDaoJdbcImpl
 * @version 1.1 : now use  DaoFactory
 */
public class AccountBean implements EntityBean {

    /** The balance */
    private double balance;
    /** The name */
    private String name;
    /** The email */
    private String email;

    /** The entity context for the bean, saved from setEntityContext */
    private EntityContext ctx;

    /** The data access object */
    private MyDao dao;


    // Business methods

    /**
    * Returns current account ID
    * @return string account ID
    */
    public String getAccountID() {
        AccountPK key = (AccountPK)ctx.getPrimaryKey();
        addItem("AccountBean.getCustID : returning ["+key.id+"]...");
        return key.id;
    }

   /**
    * Returns current balance
    * @return double balance
    */
    public double getBalance() {
        addItem("AccountBean.getBalance : returnig ["+this.balance+"]...");
        return this.balance;
    }

    /**
     * Set this Account's balance
     * @param new balance value
     */
    public void setBalance(double param) {
        addItem("AccountBean.setBalance : setting["+param+"]...");
        this.balance = param;
    }

   /**
    * Returns current name
    * @return String name
    */
    public String getName(){
        addItem("AccountBean.getName : returnig ["+this.name+"]...");
        return this.name;
    }

   /**
     * Set this Account's name
     * @param new name value
     */
    public void setName(String param){
	addItem("AccountBean.setEmail : setting name to ["+param+"]");
	this.name = param;
    }

   /**
     * Returns current email.
     * @return String email
     */
    public String getEmail(){
        addItem("getEmail : returning ["+this.email+"]");
        return this.email;
    }

    /**
     * Set this Account's email
     * @param new name value
     */
    public void setEmail(String param){
	addItem("setEmail : setting mail to ["+param+"]");
	this.email = param;
    }

    // EJB Methods

    /**
     * Create a new Account with the given ID, name, address and balance.
     * @return the primary key
     * @param custID the ID for this account
     * @param acctType the account type for this account
     * @param balance the balance for this Customer
     * @exception CreateException thrown if the custID or acctType is null,
     *   or the balance is negetive.
     */
    public AccountPK ejbCreate(String id, String name, String address, double balance)
       						throws CreateException {
        addItem("AccountBean.ejbCreate: called ...");

        if ((id == null)||(id.equals("")) || (balance < 0.0)) {
            addItem(" # ERROR ! ejbCreate : INVALID id["+id+"] or balance is negative !");
            throw new CreateException("Invalid Customer Id or balance");
        }
        try {
             // Use DAO to insert account into the underlaying layer
            AccountPK npk = new AccountPK(id);
            int count = dao.insertAccount(npk, name,address,balance);
            if (count != 1) {
                throw new CreateException("Insert of account Failed : " + count);
            }
            this.name = name;
            this.email = address;
            this.balance = balance;
	    addItem("AccountBean.ejbCreate: returning id["+id+"]");
            return npk;
        } catch (DAOException e) {
            addItem("AccountBean.ejbCreate: DAO exception while creating account: "+ e.getMessage());
            e.printStackTrace();
            throw new CreateException("Error creating Account: " + e.getMessage());
        }
    }

    /**
     * Post create settings
     * @param custID the ID for this account
     * @param acctType the account type for this account
     * @param balance the balance for this Customer
     * @exception CreateException never thrown - declared for sub-class use
     */
    public void ejbPostCreate(String id, String name, String address, double balance) throws CreateException {
        addItem("AccountBean.ejbPostCreate : called");
    }

    /**
     * Set the context for this bean : save the context in ctx member
     * @param ctx the Entity Context for this bean
     */
    public void setEntityContext(EntityContext ctx) {
        addItem("AccountBean.setEntityContext: called ...");
        this.ctx = ctx;
        try {
            // this.dao = new MyDaoJdbcImpl(DS_JNDI_NAME); no more !
            this.dao = MyDaoFactory.getDAO();
            addItem("AccountBean.setEntityContext: DAO ["+this.dao.getClass().getName()+"]...");
        } catch (DAOException e) {
            addItem("AccountBean.setEntityContext: DAO creation failed: " + e.getMessage());
            e.printStackTrace();
            throw new EJBException("DAO creation failed : " + e.getMessage());
        }
    }

    /** Unset the context for this bean. */
    public void unsetEntityContext() {
        addItem("AccountBean.usetEntityContext: called ...");
        this.dao = null;
    }

    public void ejbActivate() {
        addItem("AccountBean.ejbActivate: called ...");
    }

    public void ejbPassivate() {
        addItem("AccountBean.ejbPassivate: called ...");
    }

    /** Load this Account from the Database */
    public void ejbLoad() {

         addItem("AccountBean.ejbLoad: called ...");

         try {
            AccountPK key = (AccountPK)ctx.getPrimaryKey();
            AccountOM res = this.dao.readAccount(key);
            if (res == null) {
                addItem("AccountBean.ejbLoad: Account does not exists [" + key + "]...");
                throw new NoSuchEntityException("AccountBean.ejbLoad: does not exists " + key);
            }
            this.balance = res.getBalance();
            this.email = res.getAddress();
            this.balance = res.getBalance();

        } catch (DAOException e) {
            addItem("AccountBean.ejbLoad: DAO exception while loading Account : " + e.getMessage());
            e.printStackTrace();
            throw new EJBException("DAO exception while loading Account: " + e.getMessage());
        }
    }

    /** Store this Account to the database */
    public void ejbStore() {

        addItem("AccountBean.ejbStore: called ...");

        try {
            AccountPK key = (AccountPK)ctx.getPrimaryKey();
            int count = this.dao.updateAccount(key, name, email, balance);
            if (count == 0) {
                addItem("Account does not exists : " + key);
                throw new NoSuchEntityException("Account no longer exists: " + key);
            }
        } catch (DAOException e) {
            addItem("AccountBean.ejbStore: DAO exception while storing Account: " + e.getMessage());
            e.printStackTrace();
            throw new EJBException("DAO exception while storing Account: " + e.getMessage());
        }
    }

    /** Remove this Account from the database */
    public void ejbRemove() throws RemoveException {

        addItem("AccountBean.ejbRemove: called ...");

        try {
            AccountPK key = (AccountPK)ctx.getPrimaryKey();
            int count = this.dao.deleteAccount(key);
            if (count == 0) {
                addItem("AccountBean.ejbRemove : Account does not exists [" + key + "]...");
                throw new NoSuchEntityException("AccountBean.ejbRemove : Account does not exists " + key);
            }
        } catch (DAOException e) {
            addItem("AccountBean.ejbRemove: DAO exception while deleting Account: " + e.getMessage());
            e.printStackTrace();
            throw new EJBException("DAO exception while deleting Account: " + e.getMessage());
        }
    }

    /**
     * Find an Account
     * @param key the Account to find
     * @return the Primary Key for this Account (<code>key</code>)
     * @exception FinderException thrown if this account already exists
     */
    public AccountPK ejbFindByPrimaryKey(AccountPK key) throws FinderException {
        addItem("AccountBean.ejbFindByPrimaryKey : called...");
        try {
            boolean exists = this.dao.accountExists(key);
            if (exists) {
                return key;
            } else {
                addItem("AccountBean.ejbFindByPrimaryKey : could not find Account [" + key + "]...");
                throw new ObjectNotFoundException("AccountBean.ejbFindByPrimaryKey : could not find Account: " + key);
            }
        } catch (DAOException e) {
           addItem("AccountBean.ejbFindByPrimaryKey: DAO exception while finding Account : " + e.getMessage());
           throw new EJBException("DAO exception while finding Account: " + e.getMessage());
        }
    }

    /** Log message to console inserting date at the begin */
    private void addItem(String s){
	s = new java.util.Date().toString() + "-" + s;
	System.out.println(s);
   }

}
