package it.mokabyte.pattern.ejb.session.stateless;

import java.rmi.*;
import javax.ejb.*;
import java.sql.*;
import javax.sql.*;
import javax.naming.*;


import it.mokabyte.pattern.exception.CopyException;
import it.mokabyte.pattern.exception.DAOException;

import it.mokabyte.pattern.pk.AccountPK;

import it.mokabyte.pattern.valueobject.AccountOM;

import it.mokabyte.pattern.dao.MyDaoFactory;
import it.mokabyte.pattern.dao.MyDao;
import it.mokabyte.pattern.dao.DaoConfig;

/**
 * <p>Title: AdminBean</p>
 * <p>Description: read an account form DB, write it on XML using DaoFactory</p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author S. Rossini
 * @version 1.0
 */
public class CopyBean implements SessionBean {

  private SessionContext sessionContext;
  private static final String DS_JNDI_NAME = "java:comp/env/jdbc/MyDataBase";

  public void copyAccount(AccountPK key) throws CopyException {
        System.out.println("CopyBean.copyAccount : PK["+key.id+"]...");

        try{

         // Read from XML
         MyDao dao = MyDaoFactory.getDAO(DaoConfig.XML_TYPE);
         AccountOM obj = dao.readAccount(key);
         System.out.println("CopyBean.copyAccount : read["+obj+"]...");
         // Update to JDBC
         dao = MyDaoFactory.getDAO(DaoConfig.JDBC_TYPE);
         dao.updateAccount(key,obj.getName(),obj.getAddress(),obj.getBalance());
         System.out.println("CopyBean.copyAccount : inser done !");

        }catch(DAOException daoe){
          System.out.println(" # Exception ! CopyBean.copyAccount : " + daoe.getMessage());
          daoe.printStackTrace();
          throw new CopyException(daoe);
        }
  }

  // verify method
  public AccountOM readAccountXML(AccountPK key) throws CopyException {
        System.out.println("CopyBean.copyAccount : PK["+key.id+"]...");

        try{
         // Read from XML
         MyDao dao = MyDaoFactory.getDAO(DaoConfig.XML_TYPE);
         return dao.readAccount(key);

        }catch(DAOException daoe){
          System.out.println(" # Exception ! CopyBean.copyAccount : " + daoe.getMessage());
          daoe.printStackTrace();
          throw new CopyException(daoe);
        }
  }

  // verify method
  public AccountOM readAccountJDBC(AccountPK key) throws CopyException {
        System.out.println("CopyBean.copyAccount : PK["+key.id+"]...");

        try{
         // Read from DB
         MyDao dao = MyDaoFactory.getDAO(DaoConfig.JDBC_TYPE);
         return dao.readAccount(key);

        }catch(DAOException daoe){
          System.out.println(" # Exception ! CopyBean.copyAccount : " + daoe.getMessage());
          daoe.printStackTrace();
          throw new CopyException(daoe);
        }
  }

  public void ejbCreate() {
  }

  public void ejbRemove() throws RemoteException {
  }
  public void ejbActivate() throws RemoteException {
  }
  public void ejbPassivate() throws RemoteException {
  }
  public void setSessionContext(SessionContext sessionContext) throws RemoteException {
    this.sessionContext = sessionContext;
  }
}